#---------------------------------------
# SHARED INFO:
#
# Script: OncePerLineExtractor
# Version: 2.0.1
# Description: this script periodically reads one line at time from some text files, and write it as single line into other text files.
# Change: Usernames filter + Uppercase
# Services: Twitch, Youtube
# Overlays: None
# Update Date: 2022/07/08 - Fixed empty rows in collector file
#
#---------------------------------------
# CHANGELOG:
#
# 2019/01/29 v1.0 - Initial Release
# 2019/03/19 v1.1 - Usernames filter + Uppercase
# 2022/06/02 v2.0 - Rows collector
# 2022/07/08 v2.0.1 - Fixed empty rows in collector file
#
#---------------------------------------


#---------------------------------------
# Import Libraries
#
# This is where you will import any necessary
# libraries that you require for your Python Script.
#
#---------------------------------------
import clr, sys, json, os, codecs, time, unicodedata
clr.AddReference("IronPython.SQLite.dll")
clr.AddReference("IronPython.Modules.dll")



#---------------------------------------
# [Required] Script Information
#
# These variables are required since they will be
# used to Display Errors in the PythonScriptErrorlog.txt
# located in the Logs folder / Scripts Tab under
# settings in case a script is not behaving properly.
# This is required for both StreamlabsSystems
# & StreamlabsParameters.
#
#---------------------------------------
ScriptName = "Once Per Line Extractor"
Website = "http://www.patcha.it"
Description = "It reads one line at time from some text files, and write it as single line into other text files."
Creator = "Patcha"
Version = "2.0.1"


#---------------------------------------
# Set Variables
#
# Declare your Global Variables next.
# You will be able to use these within the Init,
# Tick and Execute functions. Once you want to edit
# a Global Variable you will need to use ex:
# global m_Response
#     m_Response = "This is the replacement"
#
#---------------------------------------
settingsFile = os.path.join(os.path.dirname(__file__), "settings.json")
CHUNK = 1024

class Settings:
    # Tries to load settings from file if given 
    # The 'default' variable names need to match UI_Config
    def __init__(self, settingsFile = None):
        if settingsFile is not None and os.path.isfile(settingsFile):
            with codecs.open(settingsFile, encoding='utf-8-sig',mode='r') as f:
                self.__dict__ = json.load(f, encoding='utf-8-sig') 
        else: #set variables if no settings file
            self.files = "'fromFile.txt'|'toFile.txt', 'fromAnother.txt'|'toAnother.txt'"
            self.timer = "5"
            self.delay = "1"
            self.uppercase = False
            self.filter = ""
            self.storeFiles = ""
            self.maxRows = ""

    # Reload settings on save through UI
    def ReloadSettings(self, data):
        self.__dict__ = json.loads(data, encoding='utf-8-sig')
        return

    # Save settings to files (json and js)
    def SaveSettings(self, settingsFile):
        with codecs.open(settingsFile,  encoding='utf-8-sig',mode='w+') as f:
            json.dump(self.__dict__, f, encoding='utf-8-sig')
        with codecs.open(settingsFile.replace("json", "js"), encoding='utf-8-sig',mode='w+') as f:
            f.write("var settings = {0};".format(json.dumps(self.__dict__, encoding='utf-8-sig')))
        return


#---------------------------------------
# [Required] Intialize Data (Only called on Load)
#
# Initialize anything that you want here.
# This function will only be called when
# the script is loaded.
#
#---------------------------------------
def Init():
    global MySettings
    global files
    global timer
    global delay
    global uppercase
    global filter
    global storeFiles
    global maxRows
    global last_time
    global row

    MySettings = Settings(settingsFile)
    last_time = time.time()

    files = checkFiles(parseFileNames(MySettings.files))
    row = [0] * len(files)

    try:
        timer = int(MySettings.timer)
    except ValueError:
        timer = 5

    try:
        delay = int(MySettings.delay)
    except ValueError:
        delay = 1

    uppercase = MySettings.uppercase

    filter = ParseFilteredNicks(MySettings.filter)

    storeFiles = checkFiles(parseFileNames(MySettings.storeFiles))

    try:
        maxRows = int(MySettings.maxRows)
    except ValueError:
        maxRows = 5

    last_time = last_time - timer

    return


#---------------------------------------
# [Required] Execute Data / Process Messages
#
# The execute function will be called when there is
# a new chat message to be processed.
# If there are no new messages then this function
# will not be called. It also receives data which you can
# process to create your own commands, games, ...
#
#---------------------------------------
def Execute(data):
    return


#---------------------------------------
# [Required] Tick Function
#
# This is the Tick function and will be executed
# every time the program progresses.
# As you can see there is no data variable here.
# So use this function when you don't require data,
# but want to do something while there is no data.
#
#---------------------------------------
def Tick():
    global last_time
    global row

    if last_time:
        now = time.time()
        if now - last_time >= timer:
            last_time = now

            # collectiong new rows first
            for i, file in enumerate(storeFiles):
                storeRow(storeFiles[i][0], storeFiles[i][1])
                if i < len(storeFiles):
                    time.sleep(delay)

            # extractiong rows after
            for i, file in enumerate(files):
                row[i] = extractRow(files[i][0], files[i][1], row[i])
                if i < len(files):
                    time.sleep(delay)

    return


def ScriptToggled(state):
    global last_time

    if state:
        last_time = time.time()
    else:
        last_time = None

    return


def parseFileNames(files):
    # i.e.: "'fromFile.txt'|'toFile.txt', 'fromAnother.txt'|'toAnother.txt'"
    files = files.split(",")
    files = [x.strip() for x in files]
    files = [x.split("|") for x in files]
    for i, couple in enumerate(files):
        for y, file in enumerate(files[i]):
            files[i][y] = files[i][y].strip().replace("'", "")
    return files


def checkFiles(files):
    for i, couple in enumerate(files):
        leng = len(files[i])
        if leng < 2:
            if leng < 1:
                files[i] = [None, None]
            else:
                files[i] = [files[i][0], None]

        if files[i][0]:
            if not os.path.isfile(files[i][0]):
                if os.path.isfile("Services\Scripts\OncePerLineExtractor\\" + files[i][0]):
                    files[i][0] = "Services\Scripts\OncePerLineExtractor\\" + files[i][0]
                else:
                    files[i][0] = None

        if files[i][1]:
            if not os.path.isfile(files[i][1]):
                try:
                    open(files[i][1], 'w').close()
                except IOError:
                    files[i][1] = "Services\Scripts\OncePerLineExtractor\\" + files[i][1]
                    try:
                        open(files[i][1], 'w').close()
                    except IOError:
                        files[i][1] = None

    return files


def ParseFilteredNicks(Words):
    # i.e.: "Nigger, Jew, Nazi, Nigga, Mother+Fucker"
    Words = Words.split(",")
    Words = [x.strip().lower() for x in Words]
    return Words

def CheckIsFilteredWord(line):
    global filter
    for word in filter:
        if word.lower() in line.lower():
            return True
    return False

def removeFirstLine(file):
    with open(file, 'r') as fin:
        data = fin.read().splitlines(True)
    fin.close()
    with open(file, 'w') as fout:
        line = data[1:]
        if line:
            fout.writelines(line)
    fout.close()
    return


def extractRow(fromFile, toFile, row):
    if fromFile and toFile:

        lines = [line.rstrip('\n') for line in open(fromFile)]

        if not row or row >= len(lines):
            row = 0
        line = lines[row]

        if uppercase:
            euroFix = False
            if u'\xe2' in line:
                euroFix = True
            line = line.upper()
            if euroFix:
                line = line.replace(u'\xc2', u'\xe2')

        if not CheckIsFilteredWord(line.lower()):
            with open(toFile, 'w') as the_file:
                the_file.write(line)

    return row + 1

def storeRow(fromFile, toFile):
    global maxRows

    if fromFile and toFile:

        lines = [line.rstrip('\n') for line in open(toFile)]
        with open(fromFile, "r") as file:
            first = file.readline().rstrip('\n')

        if first and not first in lines:
            with open(toFile, 'a') as the_file:
                the_file.write("\n" + first)

        file.close()

    if maxRows < len(lines):
        removeFirstLine(toFile)

    return

